/*
 * Copyright 2005 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"$Id: ipp.c,v 1.2 2005/11/10 17:50:58 njacobs Exp $"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <locale.h>
#include <libintl.h>
#include <papi.h>

static void
usage(char *program)
{
	char *name;

	if ((name = strrchr(program, '/')) == NULL)
		name = program;
	else
		name++;

	fprintf(stdout,
		gettext("Usage: %s [-d destination] [-T content-type]\n"),
		name);
	exit(1);
}

static struct {
	char *mime_type;
	char *lp_type;
} type_map[] = {
	{ "plain/text", "simple" },
	{ "application/octet-stream", "raw" },
	{ "application/octet-stream", "any" },
	{ "application/postscript", "postscript" },
	{ "application/postscript", "ps" },
	{ "application/x-cif", "cif" },
	{ "application/x-dvi", "dvi" },
	{ "application/x-plot", "plot" },
	{ "application/x-ditroff", "troff" },
	{ "application/x-troff", "otroff" },
	{ "application/x-pr", "pr" },
	{ "application/x-fortran", "fortran" },
	{ "application/x-raster", "raster" },
	{ NULL, NULL}
};

static char *
lp_type_to_mime_type(char *lp_type)
{
	int i;

	if (lp_type == NULL)
		return ("application/octet-stream");

	for (i = 0; type_map[i].lp_type != NULL; i++)
		if (strcasecmp(type_map[i].lp_type, lp_type) == 0)
			return (type_map[i].mime_type);

	return (lp_type);
}

int
main(int ac, char *av[])
{
	papi_status_t status;
	papi_service_t svc = NULL;
	papi_attribute_t **list = NULL;
	papi_encryption_t encryption = PAPI_ENCRYPT_NEVER;
	papi_job_t job = NULL;
	char *printer = NULL;
	int copy = 0;
	int silent = 0;
	int dump = 0;
	int validate = 0;
	int modify = -1;
	int c;

	(void) setlocale(LC_ALL, "");
	(void) textdomain("SUNW_OST_OSCMD");

	while ((c = getopt(ac, av, "T:d:st:E")) != EOF)
		switch (c) {
		case 'T':	/* type */
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
					"document-format",
					lp_type_to_mime_type(optarg));
			break;
		case 'd':	/* destination */
			printer = optarg;
			break;
		case 's':	/* suppress message */
			silent = 1;
			break;
		case 't':	/* title */
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
					"job-name", optarg);
			break;
		case 'E':
			encryption = PAPI_ENCRYPT_REQUIRED;
			break;
		default:
			usage(av[0]);
		}

	if ((printer == NULL) &&
	    ((printer = getenv("PRINTER")) == NULL) &&
	    ((printer = getenv("LPDEST")) == NULL))
		printer = "lp";

	if (modify == -1) {
		papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
				"document-format", "application/octet-stream");
	}

	status = papiServiceCreate(&svc, NULL, NULL, NULL, NULL,
					encryption, NULL);
	if (status != PAPI_OK) {
		fprintf(stderr, gettext(
			"Failed to contact service for %s: %s\n"), printer,
			papiStatusString(status));
		exit(1);
	}

	status = papiJobSubmit(svc, printer, list, NULL, &av[optind], &job);

	if (status != PAPI_OK) {
		fprintf(stderr, gettext("%s: %s\n"), printer,
			papiStatusString(status));
		exit(1);
	}

	papiAttributeListFree(list);

	if (((silent == 0) || (dump != 0)) &&
	    ((list = papiJobGetAttributeList(job)) != NULL)) {
		char *dest = "unknown";
		int32_t id = 0;

		papiAttributeListGetString(list, NULL, "printer-name", &dest);
		papiAttributeListGetInteger(list, NULL, "job-id", &id);
		printf("request id is %s-%d (%d file(s))\n", dest, id,
				ac - optind);

		if (dump != 0) {
			char buf[1024];

			buf[0] = '\0';
			papiAttributeListToString(list, "\n\t", buf,
					sizeof (buf));
			printf("job attributes:\n\t%s\n", buf);
		}
	}

	papiJobFree(job);
	papiServiceDestroy(svc);

	exit(0);
}
